
# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.2.4"

import FatxFast.gui.linkwidget as linkwidget
import FatxFast.gui.buttonwidget as buttonwidget
import FatxFast.gui.textwidget as textwidget

class SceneEditor(object):
    
    def __init__(self, caller):
        self.caller = caller
        
class ConfigSceneEditor(SceneEditor):
    
    def __init__(self, caller):
        super(ConfigSceneEditor, self).__init__(caller)
        self.config = caller.manager.game.config
        self.menuwidget = caller.get_widget("menu")
        self.scenelinks = []
        self.buttons = []
        
    def create_widgets(self):
        template = self.caller.manager.sceneparser.templates["menulink"].copy()
        template["size"] = 22
        template["height"] = 28
        for hotkey in self.config.hotkeys.keys():
            template_attributes = template
            template_attributes["name"] = hotkey
            template_attributes["href"] = "hotkeyscene"
            template_attributes["properties"] = {"setting":"hotkeys",
                                                 "button":hotkey,
                                                 "config":self.config}
            template_attributes["container"] = self.menuwidget
            template_attributes["connect_obj"] = self.caller.manager
            widget = linkwidget.SceneLinkWidget(**template_attributes)
            self.menuwidget.add_widget(widget.name, widget)
            self.scenelinks.append(widget)
            
        template = self.caller.manager.sceneparser.templates["button"].copy()
        template["size"] = 22
        template["height"] = 28
        for setting in ["fullscreen", "volume"]:
            template_attributes = template
            template_attributes["name"] = setting
            template_attributes["href"] = setting
            template_attributes["container"] = self.menuwidget
            template_attributes["connect_obj"] = self.caller
            template_attributes["handler"] = "change_setting"
            widget = buttonwidget.ButtonWidget(**template_attributes)
            self.menuwidget.add_widget(widget.name, widget)
            self.buttons.append(widget)
        template = self.caller.manager.sceneparser.templates["back"]
        template_attributes = template.copy()
        template_attributes ["container"] = self.menuwidget
        template_attributes["connect_obj"] = self.caller.manager
        template_attributes["handler"] = "pop_scene"
        widget = linkwidget.SceneLinkWidget(**template_attributes)
        self.menuwidget.add_widget(widget.name, widget)
        self.menuwidget.rect.height = (len(self.scenelinks)*45 + 
                                       len(self.buttons)*45 +
                                       45)
            
    def update_texts(self):
        for w in self.buttons:
            w.text = "%s: %s" % (w.name, str(getattr(self.config, w.name)))
        
        for w in self.scenelinks:
            w.text = "%s: %s" % (w.properties["button"],
                        self.config.get_hotkey_string(w.properties["button"]))
            
class SaveSceneEditor(SceneEditor):
    
    def create_saveboard(self):
        saves = self.caller.manager.game.savemanager.saves
        savetext = ""
        for index, save in enumerate(saves, start=1):
            try:
                date = save.game["date"]
            except (AttributeError, TypeError):
                date = "Empty"
            savetext = ("{0}: {1}".format(index, date))
            widget = self.caller.get_widget(index)
            widget.text = savetext
        self.caller.saves = saves
        
class ScoreSceneEditor(SceneEditor):
    
    def _create_score_line(self, scoretype, index, score):
        line = "{0}: {1}".format(index, score["points"])
        if scoretype == "classic":
            line += " in {0} seconds".format(score["total_time"])
        return line

    def _create_mode_section(self, scoretype, scores):
        scorelist = []
        for index, score in enumerate(scores[scoretype], start=1):
            scorelist.append(self._create_score_line(scoretype, index, score))
        return "\n".join(scorelist)

    def create_scoreboard(self):
        scores = self.caller.manager.game.hsmanager.scores
        widget = self.caller.get_widget("score")
        scorestrings = ["Total games: {0}\n".format(
            self.caller.manager.game.config.games_played)]
        for scoretype in scores:
            scoretitle = "{0} scores".format(scoretype)
            scorelist = self._create_mode_section(scoretype, scores)
            scorestrings.append(scoretitle)
            scorestrings.append(str(scorelist))
            scorestrings.append("\n")
        widget.text = "\n".join(scorestrings)

class AskLoadSceneEditor(SceneEditor):

    def connect_button_links(self):
        load_link = self.caller.get_widget("loadlink")
        load_link.connect_obj = self.caller
        load_link.handler = "load_save"
        save_link = self.caller.get_widget("savelink")
        save_link.connect_obj = self.caller
        save_link.handler = "save_to_save"

class GameEndSceneEditor(SceneEditor):

    def connect_button_links(self):
        load_link = self.caller.get_widget("restartlink")
        load_link.connect_obj = self.caller
        load_link.handler = "restart_mode"

        
        
